# Copyright © 2024 Nicole O'Connor
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
# 	http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# except where otherwise specified, all constants and their associated values are
# taken from their values observed here: https://wiki.nina.chat/wiki/Protocols/OSCAR

## foodgroups (these are not listed with FOODGROUP_ prefixes on the wiki, this is for namespace sanity)
FOODGROUP_GLOBAL        = 0x0   # not actually part of the spec (see ## global)
FOODGROUP_OSERVICE      = 0x1   # ping/pong, etc
FOODGROUP_LOCATE        = 0x2   # lookup profile data
FOODGROUP_BUDDY         = 0x3   # buddy list
FOODGROUP_ICBM          = 0x4   # inter-client messages
FOODGROUP_ADVERT        = 0x5   # buy now buy now buy now
FOODGROUP_INVITE        = 0x6   # send invite mail
FOODGROUP_ADMIN         = 0x7   # user control panel
FOODGROUP_POPUP         = 0x8   # some sort of push notification
FOODGROUP_PD            = 0x9   # permit/deny (privacy control)
FOODGROUP_USER_LOOKUP   = 0xA   # user lookup
FOODGROUP_STATS         = 0xB   # telemetry
FOODGROUP_TRANSLATE     = 0xC   # clients use this to report unsupported proto features
FOODGROUP_CHAT_NAV      = 0xD   # chat room browser
FOODGROUP_CHAT          = 0xE   # chat room messages (incl. join/parts, etc)
FOODGROUP_ODIR          = 0xF   # user lookup extension
FOODGROUP_BART          = 0x10  # buddy art (avatar, custom sounds, etc)
FOODGROUP_FEEDBAG       = 0x13  # client-server data sync
FOODGROUP_ICQ           = 0x15  # ICQ-specific extension
FOODGROUP_BUCP          = 0x17  # authN and ICQ registration
FOODGROUP_ALERT         = 0x18  # more push notifications
FOODGROUP_PLUGIN        = 0x22  # client plugin support
FOODGROUP_UNNAMED_FG_24 = 0x24  # listed on NINA wiki, likely reserved for something that never ended up making it to release
FOODGROUP_MDIR          = 0x25  # as yet undocumented. DIR implies lookup
FOODGROUP_ARS           = 0x44A # NINA extension for relay services

known_foodgroups = [
    FOODGROUP_GLOBAL,
    FOODGROUP_OSERVICE,
    FOODGROUP_LOCATE,
    FOODGROUP_BUDDY,
    FOODGROUP_ICBM,
    FOODGROUP_ADVERT,
    FOODGROUP_INVITE,
    FOODGROUP_ADMIN,
    FOODGROUP_POPUP,
    FOODGROUP_PD,
    FOODGROUP_USER_LOOKUP,
    FOODGROUP_STATS,
    FOODGROUP_TRANSLATE,
    FOODGROUP_CHAT_NAV,
    FOODGROUP_CHAT,
    FOODGROUP_ODIR,
    FOODGROUP_BART,
    FOODGROUP_FEEDBAG,
    FOODGROUP_ICQ,
    FOODGROUP_BUCP,
    FOODGROUP_ALERT,
    FOODGROUP_PLUGIN,
    FOODGROUP_UNNAMED_FG_24,
    FOODGROUP_MDIR,
    FOODGROUP_ARS
]

## global
GLOBAL__ERR             = 0x1   # 0x1 is always the ERR constant in a given foodgroup

## oservice
OSERVICE__ERR                   = GLOBAL__ERR
OSERVICE__CLIENT_ONLINE         = 0x2  # client ready
OSERVICE__HOST_ONLINE           = 0x3  # server caps
OSERVICE__SERVICE_REQUEST       = 0x4  # request a service, with params
OSERVICE__SERVICE_RESPONSE      = 0x5  # response to above, with server info
OSERVICE__RATE_PARAMS_QUERY     = 0x6  # rate limit check
OSERVICE__RATE_PARAMS_REPLY     = 0x7  # rate limit information
OSERVICE__RATE_PARAMS_SUB_ADD   = 0x8  # subscribe to rate limit change notifications for given class
OSERVICE__RATE_DEL_PARAM_SUB    = 0x9  # inverse of the above
OSERVICE__RATE_PARAM_CHANGE     = 0xA  # rate limit changes
OSERVICE__PAUSE_REQ             = 0xB  # server needs to migrate client
OSERVICE__PAUSE_ACK             = 0xC  # client understands above
OSERVICE__RESUME                = 0xD  # abort server's client migration
OSERVICE__USER_INFO_QUERY       = 0xE  # who am i
OSERVICE__USER_INFO_UPDATE      = 0xF  # you are this
OSERVICE__EVIL_NOTIFICATION     = 0x10 # user's evil level (shown in clients as a "warning level") has changed
OSERVICE__IDLE_NOTIFICATION     = 0x11 # user has gone idle
OSERVICE__MIGRATE_GROUPS        = 0x12 # here's your new server
OSERVICE__MOTD                  = 0x13
OSERVICE__SET_PRIVACY_FLAGS     = 0x14
OSERVICE__WELL_KNOWN_URLS       = 0x15 # provides service related URLs
OSERVICE__NOOP                  = 0x16 # keep alive packet, used in legacy clients (newer implementations use keep-alives at the FLAP level)
OSERVICE__CLIENT_VERSIONS       = 0x17 # client caps
OSERVICE__HOST_VERSIONS         = 0x18 # host caps
OSERVICE__MAX_CONFIG_QUERY      = 0x19 # ???
OSERVICE__MAX_CONFIG_REPLY      = 0x1A # ???
OSERVICE__STORE_CONFIG          = 0x1B # ???
OSERVICE__CONFIG_QUERY          = 0x1C # ???
OSERVICE__CONFIG_QUERY          = 0x1D # ???
OSERVICE__SET_USERINFO_FIELDS   = 0x1E # allows client to set userdata
OSERVICE__PROBE_REQ             = 0x1F # ping
OSERVICE__PROBE_ACK             = 0x20 # pong
OSERVICE__BART_REPLY            = 0x21 # bad buddy art item?
OSERVICE__BART_QUERY2           = 0x22 # get own buddy art?
OSERVICE__BART_REPLY2           = 0x23 # response to above (also sent on login)